# -*- coding: utf-8 -*-

# Copyright 2013-2014 Jorge Estrada. 
#
# This file is part of azucar.pythonUtils project. 
# It is subject to the license terms in the 
# LICENSE file found in the top-level directory of this distribution. 
# No part of azucar.pythonUtils project, including this file, may be copied,
# modified, propagated, or distributed except according to the terms contained 
# in the LICENSE file.

import subprocess
import os
import copy

c_compiler_path = "gcc"
temp_program_name = "checklib"
shell_path = "/bin/sh"


def compatibleVersions(target, actual):
    """Check if two version identifiers are compatible
    
    :param target: Identifier of the desired version
    :param actual: Identifier of the existing version
    
    :return: True if both versions are equal, False if not.
    """
    
    return (target == actual)

def getVersion(libraryName):
    """Obtain the version string from a library name.
    
    :param libraryName: The complete library name (e.g.: "libboost_iostreams-mt.so.1.44.0")
    
    :return: The version string for the libraryName. e.g.: "1.44.0" for the example above.
    
    :exception: Exception("Not a standard library name: %s") if the libraryName does not
      follow the standard format.
    :exception: Exception("Library name with empty version: %s") if the libraryName does not
      include version information.
    """
    
    found = False

    workString = libraryName

    index = workString.find("so.")

    while index != -1:
        workString = workString[index+3:]
        found = True
        index = workString.find("so.")

    if not found:
        raise Exception("Not a standard library name: %s" % libraryName)

    result = workString

    if result == "":
        raise Exception("Library name with empty version: %s" % libraryName)

    return result

def extractLibraryDependency(libraryShortName, programName):
    """Obtain the full library string required by programName
    
    :param libraryShortName: Short name for the library required by programName. I.e., for the
      math library, it is "m" and not "libm".
    :param programName: Name of the executable program or shared library that needs the
      libraryShortName.
    
    :return: The string identifying the required library and version for libraryShortName 
      (e.g.: "libboost_iostreams-mt.so.1.44.0").
    """
    
    # run objdump, grep NEEDED grep libraryShortName, then split by spaces and take second part
    output = subprocess.check_output([shell_path, "-c", "objdump -p %s | grep NEEDED | grep lib%s.so" % (programName, libraryShortName)])
    firstLine = output.splitlines()[0]

    libraryDependency = firstLine.split()[1]

    return libraryDependency

def compileWithLibrary(programName, libraryShortName):
    """Compile a test program to check if a given library is found in the system.
    
    :param programName: Name to give to the executable program created. The test program
      source file will be named like that, appending ".c".
    :param libraryShortName: Short name for the library to be tested. I.e., for the
      math library, it is "m" and not "libm".
    
    :post: An executable program programName exists, which depends on libraryShortName.
    
    :exception: Exception("Library not found.") if the program cannot be compiled due to
      the required library being missing.
    """
    
    sourceName = "%s.c" % programName

    with open(sourceName, "w") as output:
        output.write("int main() {}\n")

    try:
        newEnv = os.environ
        
        if 'LD_LIBRARY_PATH' in os.environ:
            if os.environ['LD_LIBRARY_PATH'] != "":
                newEnv = copy.deepcopy(os.environ)
                if 'LIBRARY_PATH' in newEnv:
                    if newEnv['LIBRARY_PATH'] == "":
                        newEnv['LIBRARY_PATH'] = newEnv['LD_LIBRARY_PATH']
                    else:
                        newEnv['LIBRARY_PATH'] = newEnv['LD_LIBRARY_PATH'] + os.pathsep + newEnv['LD_LIBRARY_PATH']
                else:
                    newEnv['LIBRARY_PATH'] = newEnv['LD_LIBRARY_PATH']

        retcode = subprocess.call([c_compiler_path, sourceName, "-l%s" % libraryShortName, "-o", programName],
                                  env=newEnv)

    finally:
        os.remove(sourceName)

    if retcode != 0:
        raise Exception("Library not found.")

    return

def checkLibrary(libraryShortName, targetVersion):
    """Check if a given library and specific version are available in the system.
    
    :param libraryShortName: Short name of the library. I.e.: for the math library, it should
      be 'm', instead of 'libm'.
    :param targetVersion: Required version of the library, in string form. For instance, for
      library 'libboost_iostreams-mt.so.1.44.0', it would be '1.44.0'.
    
    :return: True if the given library and specific version are available in the system.
      False if the library is available, but the version does not match.
    
    :exception: Exception("Library not found.") If the library is not found in the system.
    """
    
    compileWithLibrary(temp_program_name, libraryShortName)

    try:
        libraryDep = extractLibraryDependency(libraryShortName, temp_program_name)
    finally:
        os.remove(temp_program_name)

    libraryVersion = getVersion(libraryDep)

    if not compatibleVersions(targetVersion, libraryVersion):
        print "Expected version %s but found version %s" % (targetVersion, libraryVersion)
        return False
    return True


