# -*- coding: utf-8 -*-

# Copyright 2013-2014 Jorge Estrada. 
#
# This file is part of azucar.pythonUtils project. 
# It is subject to the license terms in the 
# LICENSE file found in the top-level directory of this distribution. 
# No part of azucar.pythonUtils project, including this file, may be copied,
# modified, propagated, or distributed except according to the terms contained 
# in the LICENSE file.

import sqlalchemy
from sqlalchemy.sql import text


def get_db_version(connection):
    """Get the version information for the connected database.

    :param connection: An SQLAlchemy Connection object, already connected
       to a database.

    :returns: A dictionary with the keys 'schema_version' (the value is
       a string with the version of the database schema) and 'script_version'
       (the value is a string codifying a UUID). If the database has no
       version information, it returns None.
    """

    # Does db_version exist?
    db_exists = not(connection.execute(text("SHOW TABLES LIKE 'DbVersion'")).scalar() is None)
    
    if db_exists:
        result = connection.execute(text("SELECT schema_version, script_version FROM DbVersion"))
        row = result.fetchone()
        return_value = {'schema_version': row['schema_version'],
                        'script_version': row['script_version']}
        result.close()
    else:
        return_value = None

    return return_value

def connect_to_database(user, password, db):
    """Connect to the localhost MySQL DBMS and a specific database. 

    :param user: DBMS user name.
    :param password: DBMS user password.
    :param db: Database name.

    :returns: An SQLAlchemy Engine object representing the connection to
       the requested database with the given user and password.
    """

    return sqlalchemy.create_engine('mysql+pyodbc://%(user)s:%(password)s@localhost/%(db)s'
                                    % {'user': user, 'password': password, 'db': db})

def get_tables(connection):
    """Get a list of all the tables in the connected database.

    :param connection: An SQLAlchemy Connection object, representing a connection
      to a database.

    :returns: A list of strings, where each one is the name of a table in the
      connected database, representing all the tables in that database.
    """

    list_of_tables = []

    result = connection.execute(text("SHOW TABLES"))

    for row in result:
        list_of_tables.append(row[0])

    result.close()

    return list_of_tables

def drop_table(connection, table_name):
    """Drop a table in a connected database.

    :param connection: An SQLAlchemy Connection object, representing a connection
      to a database.
    :param table_name: A string with the name of the table to drop.
    """

    result = connection.execute(text("DROP TABLE IF EXISTS %s" % table_name))
    result.close()

