# -*- coding: utf-8 -*-

# Copyright 2013-2014 Jorge Estrada. 
#
# This file is part of azucar.pythonUtils project. 
# It is subject to the license terms in the 
# LICENSE file found in the top-level directory of this distribution. 
# No part of azucar.pythonUtils project, including this file, may be copied,
# modified, propagated, or distributed except according to the terms contained 
# in the LICENSE file.

import os
import shutil
import sys

import azucar.util


def copyFile(sourcePath, targetDir, user, group, permissions, overwrite=False):
    """Copy a file to a destination directory, setting its ownership to user and group,
    and setting its access permissions.
    
    :param sourcePath: Path of the file to copy.
    :param targetDir: Path of the target directory.
    :param user: User id to assign ownership of the target file.
    :param group: Group id to assign ownership of the target file.
    :param permissions: Octal file access permissions for the target file.
    :param overwrite: If True, then overwrite any previousely existing target file. If False,
      copy if the target file does not already exist, and raise and exception if it does exist.
    
    :exception: Exception If the target file already exists and overwrite is False.
    """
    
    sourceFilename = os.path.basename(sourcePath)
    targetPath = os.path.join(targetDir, sourceFilename)
    if os.path.exists(targetPath) and not overwrite:
        raise Exception("Target '%s' file exists, but you have not selected neither the overwrite nor the clean-install options." % (targetPath,))
    
    if not os.path.exists(os.path.dirname(targetPath)):
        os.makedirs(os.path.dirname(targetPath))
        
    shutil.copy(sourcePath, targetPath)
    os.chown(targetPath, user, group)
    os.chmod(targetPath, permissions)

def copyFile2(sourcePath, targetPath, user, group, permissions, overwrite=False):
    """Copy a file to a destination location, setting its ownership to user and group,
    and setting its access permissions.
    
    :param sourcePath: Path of the file to copy.
    :param targetPath: Path of the target file.
    :param user: User id to assign ownership of the target file.
    :param group: Group id to assign ownership of the target file.
    :param permissions: Octal file access permissions for the target file.
    :param overwrite: If True, then overwrite any previousely existing target file. If False,
      copy if the target file does not already exist, and raise and exception if it does exist.
    
    :exception: Exception If the target file already exists and overwrite is False.
    """
    
    if os.path.exists(targetPath) and not overwrite:
        raise Exception("Target '%s' file exists, but you have not selected neither the overwrite nor the clean-install options." % (targetPath,))
    
    if not os.path.exists(os.path.dirname(targetPath)):
        os.makedirs(os.path.dirname(targetPath))
        
    shutil.copy(sourcePath, targetPath)
    os.chown(targetPath, user, group)
    os.chmod(targetPath, permissions)

def copyFileWithSubstitution2(sourcePath, targetPath, user, group, permissions, mappings,
                              overwrite=False):
    """Copy a file to a destination location, setting its ownership to user and group,
    and setting its access permissions, and replacing string.Template variables with the
    values given in mappings.
    
    :param sourcePath: Path of the file to copy.
    :param targetPath: Path of the target file.
    :param user: User id to assign ownership of the target file.
    :param group: Group id to assign ownership of the target file.
    :param permissions: Octal file access permissions for the target file.
    :param mappings: Dictionary indexed by string.Template substitution variables to replace,
      in the file copied to targetPath, with the values those variables index in the dictionary.
    :param overwrite: If True, then overwrite any previousely existing target file. If False,
      copy if the target file does not already exist, and raise and exception if it does exist.
    
    :exception: Exception If the target file already exists and overwrite is False.
    """
    

    if os.path.exists(targetPath) and not overwrite:
        raise Exception("Target '%s' file exists, but you have not selected neither the overwrite nor the clean-install options." % (targetPath,))
    
    if not os.path.exists(os.path.dirname(targetPath)):
        os.makedirs(os.path.dirname(targetPath))
                
    azucar.util.customizeTemplateFile(sourcePath, targetPath, mappings)

    os.chown(targetPath, user, group)
    os.chmod(targetPath, permissions)

def createFile(targetPath, user, group, permissions, overwrite=False):
    """Create an empty file at the targetPath, setting its ownership to user and group,
    and setting its access permissions.
    
    :param targetPath: Path of the file to create.
    :param user: User id to assign ownership of the target file.
    :param group: Group id to assign ownership of the target file.
    :param permissions: Octal file access permissions for the target file.
    :param overwrite: If True, then overwrite any previously existing file. If False,
      create it if the file does not already exist, and raise and exception if it does exist.
    
    :exception: Exception If the file already exists and overwrite is False.
    """
    
    if os.path.exists(targetPath) and not overwrite:
        raise Exception("Target '%s' file exists, but you have not selected neither the overwrite nor the clean-install options." % (targetPath,))

    if not os.path.exists(os.path.dirname(targetPath)):
        os.makedirs(os.path.dirname(targetPath))

    touchFile = open(targetPath, 'w')
    touchFile.close() 

    os.chown(targetPath, user, group)
    os.chmod(targetPath, permissions)
    
def createDir(targetPath, user, group, overwrite=False):
    """Create directory targetPath, setting its ownership to user and group.
    
    :param targetPath: Path of the directory to create.
    :param user: User id to assign ownership of the target directory.
    :param group: Group id to assign ownership of the target directory.
    :param overwrite: If True, then change ownership of any previously existing directory. If False,
      create the directory does not already exist, and raise and exception if it does exist.
    
    :exception: Exception If the directory already exists and overwrite is False, or if
      the directory exists but is not a directory file.
    
    :post: If any directory in the route to targetPath does not exist, it is created (though
      the ownership is not changed to user and group).
    """
    
    if os.path.exists(targetPath) and not overwrite:
        raise Exception("Target '%s' directory exists, but you have not selected neither the overwrite nor the clean-install options." % (targetPath,))

    if os.path.exists(targetPath) and (not os.path.isdir(targetPath)):
        raise Exception("Target '%s' file exists, but is not a directory" % (targetPath,))

    if not os.path.exists(targetPath):
        os.makedirs(targetPath)

    os.chown(targetPath, user, group)
