# -*- coding: utf-8 -*-

# Copyright 2013-2014 Jorge Estrada. 
#
# This file is part of azucar.pythonUtils project. 
# It is subject to the license terms in the 
# LICENSE file found in the top-level directory of this distribution. 
# No part of azucar.pythonUtils project, including this file, may be copied,
# modified, propagated, or distributed except according to the terms contained 
# in the LICENSE file.

import math

class Unit:
  def __init__(self, symbol):
    self.symbol = symbol

  def __eq__(self, other):
    assert(isinstance(other, Unit))
    
    return self.symbol == other.symbol

  def __ne__(self, other):
    return not (self == other)

  def __hash__(self):
    return hash(self.symbol)

class Prefix:
  def __init__(self, symbol):
    self.symbol = symbol

  def __eq__(self, other):
    assert(isinstance(other, Prefix))
    
    return self.symbol == other.symbol

  def __ne__(self, other):
    return not (self == other)

  def __hash__(self):
    return hash(self.symbol)


class Quantity:
  def __init__(self, value, unit):
    self.value = value
    self.unit = unit
  
  def __lt__(self, other):
    assert(isinstance(other, Quantity))
    assert(self.unit == other.unit)
    
    return self.value < other.value
  
  def __eq__(self, other):
    assert(isinstance(other, Quantity))
    assert(self.unit == other.unit)
    
    return self.value == other.value
  
  def __ne__(self, other):
    return not (self == other)
  
  def __lte__(self, other):
    return (self < other) or (self == other)
  
  def __gt__(self, other):
    return not (self <= other)
  
  def __ge__(self, other):
    return not (self < other)
  
  def __hash__(self):
    return hash(self.value) ^ hash(self.unit)
    
def createDerivedUnit(prefix, baseUnit):
  newSymbol = prefix.symbol + baseUnit.symbol
  return Unit(newSymbol)



Celsius = Unit("Celsius")
Kelvin = Unit("Kelvin")
Joule = Unit("Joule")
kJperMole = Unit("kiloJoulePerMole")
kJperMoleSq = Unit("(kiloJoulePerMole)^2")
kJperMolepere2 = Unit("kiloJoulePerMolePere^2");
kcalperMole = Unit("kiloCalPerMole")
kcalperMoleSq = Unit("(kiloCalPerMole)^2")
Molar = Unit("Molar")
# boltzmann k times Temperature
kT = Unit("kT")
# boltzmann k times Temperature per elementary charge
kTpere = Unit("kTpere")
# boltzmann k times Temperature per elementary charge squared
kTpere2 = Unit("kTpere^2")
# elementary charge
eCharge = Unit("e")
Coulomb = Unit("C")
angstrom = Unit("Angstrom")
meter = Unit("meter")
kcalperMoleKelvin = Unit("kiloCalPerMoleKelvin")
kJperMoleKelvin = Unit("kiloJoulePerMoleKelvin")
jouleperMoleKelvin = Unit("JoulePerMoleKelvin")
jouleperKelvin = Unit("JoulePerKelvin")
faradperMeter = Unit("FaradPerMeter")
meterperFarad = Unit("meterPerFarad")
moleToMinusOne = Unit("mol^-1")
# meadPotentialUnit is an energy unit in the following
# units:
# e^2 * k_e /(Angstrom) where k_e is the Coulomb constant.
meadPotentialUnit = Unit("e^2*k_eperAngstrom")

allUnits = [Celsius, Kelvin, Joule, kJperMole, kJperMoleSq,
            kcalperMole, 
            kcalperMoleSq, Molar,
            kT, kTpere, kTpere2, eCharge, Coulomb,
            angstrom, meter,
            kcalperMoleKelvin, kJperMoleKelvin,
            jouleperMoleKelvin, jouleperKelvin,
            faradperMeter, meterperFarad, 
            moleToMinusOne]

unitsDict = {}

for unit in allUnits:
  unitsDict[unit.symbol] = unit


nist_MolarGasConstant = Quantity(8.314472, jouleperMoleKelvin)
nist_ElectricConstant = Quantity(8.854187817E-12, faradperMeter)
nist_AvogadroConstant = Quantity(6.02214179E+23, moleToMinusOne)
nist_BoltzmannConstant = Quantity(1.3806504E-23,jouleperKelvin)
conversion_OneCalorie = Quantity(4.184, Joule)
conversion_OneeCharge = Quantity(1.602176487E-19,Coulomb)
conversion_OneAngstrom = Quantity(1.0E-10,meter)

absolute_zero_in_Celsius = Quantity(-273.15, Celsius)


constant_R_in_kcal = Quantity(nist_MolarGasConstant.value / (1000 * conversion_OneCalorie.value),
                              kcalperMoleKelvin)
constant_R_in_kJ = Quantity(nist_MolarGasConstant.value / 1000,
                              kJperMoleKelvin)
coulomb_constant = Quantity(1.0 / (4.0 * math.pi * nist_ElectricConstant.value),
                            meterperFarad)

conversion_OneMeadPotentialUnit = Quantity(coulomb_constant.value * (conversion_OneeCharge.value**2) \
                                           / conversion_OneAngstrom.value, Joule)

def toKelvinScale(temperatureInCelsius):
  assert(temperatureInCelsius.unit == Celsius)
  
  return Quantity(temperatureInCelsius.value - absolute_zero_in_Celsius.value,
                  Kelvin)


# to convert from mead potential units to kcal/mole, use the formula:
# conversion_OneMeadPotentialUnit.value * nist_AvogadroConstant.value / (conversion_OneCalorie.value * 1000)
