# -*- coding: utf-8 -*-

# Copyright 2013-2014 Jorge Estrada. 
#
# This file is part of azucar.pythonUtils project. 
# It is subject to the license terms in the 
# LICENSE file found in the top-level directory of this distribution. 
# No part of azucar.pythonUtils project, including this file, may be copied,
# modified, propagated, or distributed except according to the terms contained 
# in the LICENSE file.

from __future__ import with_statement


import sys
import math
import os
from string import Template
from contextlib import contextmanager



class MyException(Exception):

    """Base class for user exceptions."""
    
    def __init__(self, value):
        """Create a new exception instance.
        
        :param value: Value to associate to this exception instance.
        """
        self.value = value
    def __str__(self):
        return repr(self.value)

#float_eps = sys.float_info.epsilon
float_eps = 0.000001
"""maximum error admitted in equality comparisons."""

max_float = sys.float_info.max
"""maximum allowed float value."""

min_float = sys.float_info.min
"""minimum allowed float value.""" 

def floatEqual(a, b):
    """Determine if 2 float values might be considered equal.
    
    Implements the algorithm of [squassabia:2000]
    (Squassabia, A., "Comparing Floats: How To Determine if Floating Quantities Are Close Enough Once a Tolerance Has Been Reached",
    Application Development Trends, 2000).
    
    :param a: One of the 2 values to compare.
    :param b: The other of the 2 values to compare.
    
    :returns: True if `a` is approximately equal to `b`. False otherwise.
    
    For tolerance, it uses :py:data:`float_eps`

    """
    afabs = math.fabs(a)
    bfabs = math.fabs(b)
    
    if ( (afabs < 1.0) and (bfabs > afabs * max_float)):
        return False

    if ( (afabs > 1.0) and (bfabs < a * min_float)):
        return False

    return ((1 - float_eps) < (b/a)) and ((b/a) < (1 + float_eps))


class CustomTemplate(Template):
    """A specialization of string.Template that allows identifiers with dots (see :py:attr:`idpattern`).
    
    """

    # Though in lowercase, it also accepts upper case identifiers
    idpattern = '[._a-z][._a-z0-9]*'
    """Though in lowercase, it also accepts upper case identifiers."""

def customizeTemplateFile(inputFilename, outputFilenameOrDir, translateDict):
    """Create a file from inputFilename substituting the variables in translateDict.
    
    
    :pre: Access to inputFilename for reading and to outputFilename for writing is allowed.
    :post: outputFilenameOrDir (or outputFilenameOrDir/basename(inputFilename)),
     is a copy of inputFilename, with its placeholders substituted
     according to the translateDict mapping. Valid placeholder names are those of
     CustomTemplate. Substitution is based on string.Template rules.
    
    :param inputFilename: path to the input text file.
    :param outputFilename: path to the output text file to be created, or to the output directory
      where to create the output file (with the same basename as inputFilename).
    :param translateDict: any dictionary-like object with keys that match the placeholders in the template text of the input file.
     
    """    
    if os.path.isdir(outputFilenameOrDir):
        outputFilename = os.path.join(outputFilenameOrDir, os.path.basename(inputFilename))
    else:
        outputFilename = outputFilenameOrDir
    with open(inputFilename, "r") as input:
      completeTemplate = CustomTemplate(input.read())
      finalText = completeTemplate.safe_substitute(translateDict)
      with open(outputFilename, "w") as output:
        output.write(finalText)

@contextmanager
def cd(newdir):
    """Change directory and, after executing the with block, return to the current dir.
    
    
    Sample execution::
    
        with cd(newdir):
            do work

    """
    
    olddir = os.getcwd()

    # this must be executed outside the try block since, if it 
    # throws, no chdir to olddir is needed
    os.chdir(newdir)

    try:
        yield
    finally:
        os.chdir(olddir)

def search_in_environ_path(name):
    """Get the path to file name using the environment variable PATH as a search list of
    directories.
    
    :param name: The name of the file to search.
    :return: The path to the file name, if found using the PATH variable. First directory
       that matches is used. If PATH does not exist, or the file is not found, return None.
    
    """
    
    path_variable = os.getenv('PATH')
    
    if path_variable:
        for dir in path_variable.split(os.pathsep):
            path_to_test = os.path.join(dir, name)
            if os.path.exists(path_to_test):
                return path_to_test
    
    return None

def find_script_or_raise(script_name):
    """Get the path to file script_name using the environment variable PATH as a search list of
    directories. Raise MyException if not found.
    
    :param script_name: The name of the file to search.
    
    :return: The absolute path to the file searched, or raise MyException if not found.
    """
    
    script_path = search_in_environ_path(script_name)

    if script_path is None:
        raise MyException("Could not find %s script." % script_name)
    
    return script_path

