#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright 2013-2014 Jorge Estrada. 
#
# This file is part of azucar.pythonUtils project. 
# It is subject to the license terms in the 
# LICENSE file found in the top-level directory of this distribution. 
# No part of azucar.pythonUtils project, including this file, may be copied,
# modified, propagated, or distributed except according to the terms contained 
# in the LICENSE file.

import argparse
import glob
import os
import imp
import json
import sys

from azucar.db_utils import get_db_version, connect_to_database

import azucar.util

version_string = azucar.util.__version__

script_counter = 1

scripts_dict = {}

init_script = None
last_script = None

def get_uuid_from_script_name(script_filename):
    """Extract the UUID encoded in the name of a script file.

    The format of script_filename is
    UUID_description.db.py

    where UUID is in 32-hex format with optional dashes grouping the digits,
    and description is a description text without spaces but which can use
    underscores. Example:

      22a6a95f-8270-47ff-9543-a3a1a2911dd9_initialization.db.py

    :param script_filename: Name of a database script file.

    :returns: The UUID encoded in the name of script_filename.
    
    """

    return script_filename.split("_")[0]

def get_next_module_name():
    """Get a new module name to use for a DB script.

    The name must be different from any other DB script module name already assigned.

    :returns: A string representing the module name.

    """

    global script_counter
    script_counter += 1
    return "script%d" % script_counter

class DBScriptFile(object):
    """Information related to a DB script file.

    Fields are:
    
    - previous: link to a DBScriptFile corresponding to the immediate
       predecessor for this DBScriptFile.
    - previous_id: UUID string corresponding to the identifier of the
       DB script file immediate predecessor of this.
    - next: link to the DBScriptFile corresponding to the immediate
      successor for this DBScriptFile.
    - id: UUID string identifying this script.
    - filepath: File path of the python script file this object represents.
    - module: Python module object corresponding to this script.
    - module_name: Name of the Python module corresponding to this script.

    """

    def __init__(self, filepath, **kwargs):
        """Create an uninitialized DBScriptFile corresponding to filepath.

        :param filepath: File path of the python script file this object represents.
        """

        super(DBScriptFile, self).__init__(**kwargs)
        self.previous = None
        self.previous_id = None
        self.next = None
        self.id = None
        self.filepath = filepath
        self.module = None
        self.module_name = None

    def load(self):
        """Load this script file as a module.

        Post
        ----

        After execution, self.module_name, self.module, and self.id are initialized.
        """

        self.module_name = get_next_module_name()
        self.module = imp.load_source(self.module_name, self.filepath)
        self.id = self.module.IDENTIFIER

        previous_script = self.module.PREVIOUS_SCRIPT
        if previous_script:
            self.previous_id = get_uuid_from_script_name(previous_script)

def is_predecessor(uuid_a, uuid_b):
    """Find out if uuid_a identifies a DBScriptFile in scripts_dict
    which is a predecessor of uuid_b.

    :returns: Boolean.
    """

    script = scripts_dict[uuid_a]

    if script.next:
        script = script.next
        while script:
            if script.id == uuid_b:
                return True
            script = script.next
        return False
    else:
        return False

def find_script_with_previous_id(previous_id):
    """Find a DBScriptFile in scripts_dict whose predecessor script is 
    identified by previous_id.

    :param previous_id: UUID string identifying the predecessor script.

    :returns: A DBScriptFile whose predecessor script is identified by
      previous_id, or None if not such object is found in scripts_dict.
    """

    for script in scripts_dict.values():
        if script.previous_id == previous_id:
            return script

    return None

def add_script(pathname):
    """Add the DBScriptFile corresponding to pathname to scripts_dict.

    If its predecessor or next script are already in scripts_dict, link them
    to this one.

    If it is the first script of the chain (that is, it has no predecessor),
    assign it to init_script.

    :param pathname: File path of the python script file to process.
    """

    global init_script

    script = DBScriptFile(pathname)

    script.load()

    if script.previous_id:
        if script.previous_id in scripts_dict:
            script.previous = scripts_dict[script.previous_id]
            scripts_dict[script.previous_id].next = script
    else:
        assert(init_script is None)
        init_script = script

    next_script = find_script_with_previous_id(script.id)

    if next_script:
        script.next = next_script
        next_script.previous = script

    scripts_dict[script.id] = script

def build_revisions_list(scripts_dir):
    """Populate scripts_dir with all the DB scripts in scripts_dir.

    Initialize init_script and last_script, and link each DBScriptFile to
    its predecessor and to its successor.
    """

    global last_script

    for filename in glob.glob(os.path.join(scripts_dir, "*.db.py")):
        add_script(filename)

    # set last_script
    script = init_script

    while script.next:
        script = script.next

    last_script = script

def main(args):
    connection_data = read_json_configuration_file(args.config_file)

    # Add current dir to sys.path, so that modules needed by 
    # the database scripts are found
    sys.path.append(os.path.abspath('.'))

    build_revisions_list('.')

    db_engine = connect_to_database(connection_data["user"], 
                                    connection_data["passwd"], 
                                    connection_data["db"])

    conn = db_engine.connect()

    db_version = get_db_version(conn)

    if db_version:
        source_uuid = db_version['script_version']
    else:
        source_uuid = None

    if args.target_revision_script:
        target_uuid = get_uuid_from_script_name(args.target_revision_script)
    else:
        target_uuid = last_script.id 

    if source_uuid:
        source_precedes_target = is_predecessor(source_uuid, target_uuid)
        script = scripts_dict[source_uuid]
    else:
        source_precedes_target = True
        script = None
    

    target_script_executed = (source_uuid == target_uuid)

    while not target_script_executed:
        # print script.filepath
        if source_precedes_target:
            # script.next can never be None since it would have
            # first matched script.id == target_uuid before entering here
            if script:
                next = script.next
            else:
                next = init_script

            print next.filepath
            with conn.begin() as trans:
                next.module.upgrade(conn)
                next.module.stamp_version(conn)

            script = next

            if script.id == target_uuid:
                target_script_executed = True
        else:
            # script.previous can never be None since it would have
            # first matched script.id == target_uuid before entering here
            previous = script.previous

            print script.filepath
            with conn.begin() as trans:
                script.module.downgrade(conn)
                previous.module.stamp_version(conn)

            script = previous
                
            if script.id == target_uuid:
                target_script_executed = True


def read_json_configuration_file(filename):
    with open(filename, "r") as input:
        result = json.load(input, encoding='utf-8')

    return result

def configureArgumentParser():
  parser = argparse.ArgumentParser(description='Setup database schema to a given revision.')
  parser.add_argument('--version', action='version', version=version_string)
  parser.add_argument('--config_file', help='JSON file with the database connection configuration.', default='db_config.json')
  parser.add_argument('target_revision_script', help='File name of the DB upgrade script whose schema revision is desired. If not provided, the latest revision is used.', nargs='?', default=None)

  return parser


if __name__ == "__main__":
    parser = configureArgumentParser()
    args = parser.parse_args()

    main(args)
